diff -urNX /home/leigh/.diffex linuxppc_2_4_devel-prev/arch/ppc/boot/prep/head.S linuxppc_2_4_devel-local/arch/ppc/boot/prep/head.S
--- linuxppc_2_4_devel-prev/arch/ppc/boot/prep/head.S	Sun Sep  8 11:45:22 2002
+++ linuxppc_2_4_devel-local/arch/ppc/boot/prep/head.S	Sun Sep  8 17:43:47 2002
@@ -44,10 +44,28 @@
 	isync
 
 	mr	r11,r3		/* Save pointer to residual/board data */
+/*
+ * Save the OF pointer to r25, but only if the entry point is in a sane
+ * location; if not we store 0.  If there is no entry point, or it is
+ * invalid, we establish the default MSR value immediately.  Otherwise,
+ * we defer doing that, to allow OF functions to be called, until we
+ * begin uncompressing the kernel.
+ */
+	lis	r3,0x0fff		/* r3 = 0x0fffffff */
+	ori	r3,r3,0xffff
+
+	subc	r3,r3,r5		/* r3 = (r5 <= r3) ? ~0 : 0 */
+	subfe	r3,r3,r3
+	nand	r3,r3,r3
 
-	/* Establish default MSR value */
-	li	r3,MSR_IP|MSR_FP
+	and.	r5,r5,r3		/* r5 will be cleared if (r5 > r3) */
+	bne+	haveOF
+
+	li	r3,MSR_IP|MSR_FP	/* Not OF: set MSR immediately */
 	mtmsr	r3
+	isync
+haveOF:
+	mr	r25,r5
 
 	/* compute the size of the whole image in words. */
 	lis	r4,cmd_buf@h
@@ -121,6 +139,7 @@
 	mr	r4,r7			/* Program length */
 	mr	r5,r6			/* Checksum */
 	mr	r6,r11			/* Residual data */
+	mr	r7,r25			/* Validated OF pointer */
 	bl	decompress_kernel
 
 	/*
diff -urNX /home/leigh/.diffex linuxppc_2_4_devel-prev/arch/ppc/boot/prep/misc.c linuxppc_2_4_devel-local/arch/ppc/boot/prep/misc.c
--- linuxppc_2_4_devel-prev/arch/ppc/boot/prep/misc.c	Sun Sep  8 11:45:22 2002
+++ linuxppc_2_4_devel-local/arch/ppc/boot/prep/misc.c	Sun Sep  8 17:39:02 2002
@@ -23,6 +23,7 @@
 
 #include "mpc10x.h"
 #include "nonstdio.h"
+#include "of1275.h"
 #include "zlib.h"
 
 /*
@@ -110,7 +111,7 @@
 
 unsigned long
 decompress_kernel(unsigned long load_addr, int num_words, unsigned long cksum,
-		  RESIDUAL *residual)
+		  RESIDUAL *residual, void *promptr)
 {
 	int timer = 0;
 	extern unsigned long start;
@@ -121,6 +122,10 @@
 	int start_multi = 0;
 	unsigned int pci_viddid, pci_did, tulip_pci_base, tulip_base;
 
+	/* If we have Open Firmware, initialise it immediately */
+	if (promptr)
+		ofinit(promptr);
+
 	serial_fixups();
 #if defined(CONFIG_SERIAL_CONSOLE)
 	com_port = serial_init(0, NULL);
@@ -282,6 +287,18 @@
 	*cp = 0;
 	puts("\nUncompressing Linux...");
 
+	/*
+	 * If we have OF, then we have deferred setting the MSR.
+	 * We must set it now because we are about to overwrite
+	 * the exception table.  The new MSR value will disable
+	 * machine check exceptions and point the exception table
+	 * to the ROM.
+	 */
+	if (promptr) {
+		mtmsr(MSR_IP | MSR_FP);
+		asm volatile("isync");
+	}
+
 	gunzip(0, 0x400000, zimage_start, &zimage_size);
 	puts("done.\n");
 
